@@ -37,42 +37,30 @@ html = (
 "<p>Two or more newlines are reduced to a paragraph start/stop</p>"
 "<p>All other whitespace reduced to one space</p>"
 "<h3>Block-level (hard left)</h3>"
+"<h4>Attributes</h4>"
+"<i>mark</i>HTMLtag,class(href)\"text\""
+"<h4>Marks</h4>"
 "<pre><code>"
 "= headline\n"
 "==== headline (four-deep)\n"
-"</code></pre>"
-"<pre><code>"
 "++\n"
 "- list item\n"
 "+\n"
-"</code></pre>"
-"<pre><code>"
-"+ol\n"
-"- ordered list item\n"
-"+\n"
-"</code></pre>"
-"<pre><code>"
 "+dl\n"
 "~ term\n"
 ": definition\n"
 "+\n"
-"</code></pre>"
-"<pre><code>"
 ">>\n"
 "    blockquote\n"
 ">\n"
-"</code></pre>"
-#"<pre><code>"
-#"*{image-address} image caption*\n"
-#"</code></pre>"
-"<pre><code>"
+"*(image-address)\"image caption\*\n"
 "??\n"
-"    unformatted\n"
+"    escaped\n"
 "?\n"
 "</code></pre>"
 "<h3>Inline-level (in text)</h3>"
 "<pre><code>"
-"[a{link address} link text]\n"
+"{a(link address) link text}\n"
 "</code></pre>"
 )
 
